local thunderbolts = {}
local rnd_xy = 100
local sounds = {}
local sky_brightness = 0

local prev_lifetime_time = 0
local light_multiplier = 2
local spawn_time_min = 1 -- these two are low because the bolt doesn't often find a point it can connect with
local spawn_time_max = 2

function rnd(mi, ma) return math.floor(math.random(mi, ma)) end
function rndf(mi, ma) return math.random() + math.random(mi, ma) end
function len(tbl) local i = 0; for _, v in pairs(tbl) do i = i + 1 end; return i end

local function generate_branches(start)
    local branches = {}
    local branch_count = rnd(8, 24)

    for i = 1, branch_count do
        local branch = {
            start = {},
            ending = {
                0, 0, 0
            }
        }
        
        if i == 1 then
            branch.start = {
                start[1],
                start[2],
                start[3],
            }
        else
            branch.start = branches[i - 1].ending
        end

        local dir = {
            rnd(0, 1),
            rnd(0, 1),
            rnd(0, 1),
        }

        local prev_branch = branches[i - 1]
        if prev_branch then
            local prev_dir = prev_branch.dir
            if prev_dir[1] == 0 then
                branch.ending[1] = branch.start[1] - rndf(1, 2)
            elseif prev_dir[1] == 1 then
                branch.ending[1] = branch.start[1] + rndf(1, 2)
            end

            branch.ending[2] = branch.start[2] - rndf(1, 2)

            if prev_dir[3] == 0 then
                branch.ending[3] = branch.start[3] - rndf(1, 2)
            elseif prev_dir[3] == 1 then
                branch.ending[3] = branch.start[3] + rndf(1, 2)
            end
        else
            branch.ending[1] = branch.start[1] + rnd(1, 2)
            branch.ending[2] = branch.start[2] - rnd(1, 2)
            branch.ending[3] = branch.start[3] + rnd(1, 2)
        end

        branch.dir = dir

        table.insert(branches, branch)
    end

    return branches
end

local function add_thunderbolt(start, ending)
    local hit, point, normal, shape = QueryClosestPoint(ending, 10)
    if not hit then return end

    local t = TransformToParentPoint(start, point)
    local dist = math.sqrt(
        math.pow(t[1] - start[1], 2) +
        math.pow(t[2] - start[2], 2) +
        math.pow(t[3] - start[3], 2)
    )

    local id = #thunderbolts + 1
    local bolt = {
        start = point,
        lines = {},
        branches = {},
        life = rnd(1, 2)
    }

    local player_pos = GetPlayerTransform().pos
    local y = 0

    -- generate lines
    for i = 1, dist do
        if i == 1 then
            bolt.lines[1] = {}
            bolt.lines[1].start = bolt.start
            bolt.lines[1].ending = {bolt.start[1], bolt.start[2] + 1, bolt.start[3]}
        else
            bolt.lines[i] = {}
            bolt.lines[i].start = bolt.lines[i - 1].ending

            local x = rnd(bolt.lines[i - 1].ending[1] - 1, bolt.lines[i - 1].ending[1] + 1)
            local y = bolt.lines[i - 1].ending[2] + 1
            local z = rnd(bolt.lines[i - 1].ending[3] - 1, bolt.lines[i - 1].ending[3] + 1)

            bolt.lines[i].ending = {x, y, z}

            local s_line = bolt.lines[i - 1].start
            SpawnFire(s_line)

            local pdist = math.sqrt(
                math.pow(player_pos[1] - s_line[1], 2) +
                math.pow(player_pos[2] - s_line[2], 2) +
                math.pow(player_pos[3] - s_line[3], 2)
            )

            if pdist <= 1 then
                SetPlayerHealth(0)
            end

            local gen_branch_chance = rnd(0, 100)
            if gen_branch_chance >= 99 then
                local branch = generate_branches(bolt.lines[i].ending)
                table.insert(bolt.branches, branch)
            end

            table.insert(bolt.lines, bolt.lines[i])
        end
    end

    SetEnvironmentProperty("skyboxbrightness", 30)

    table.insert(thunderbolts, bolt)

    local rand_sound = rnd(1, #sounds)
    PlaySound(sounds[rand_sound])
end

local function debug()
    DebugWatch("pos", GetPlayerCameraTransform().pos)
    DebugWatch("thunderbolts", #thunderbolts)
end

local next_hit = 0
function init()
    sounds[#sounds + 1] = LoadSound("MOD/snd/thunder1.ogg")
    sounds[#sounds + 1] = LoadSound("MOD/snd/thunder2.ogg")
    sounds[#sounds + 1] = LoadSound("MOD/snd/thunder3.ogg")

    sky_brightness = GetEnvironmentProperty("skyboxbrightness")
    next_hit = rnd(spawn_time_min, spawn_time_max)
end

function tick()
    local rain = GetEnvironmentProperty("rain")

    if #thunderbolts == 0 then
        local cur_skybox_brightness = GetEnvironmentProperty("skyboxbrightness")
        if sky_brightness ~= cur_skybox_brightness then
            sky_brightness = cur_skybox_brightness
        end
    end

    if rain <= 0.2 then return end

    local player_pos = GetPlayerTransform().pos

    -- generate thunderbolts
    local time = GetTime()
    if time >= next_hit then
        local x1 = rnd(player_pos[1] - rnd_xy, player_pos[1] + rnd_xy)
        local y1 = rnd(400, 750)
        local z1 = rnd(player_pos[3] - rnd_xy, player_pos[3] + rnd_xy)

        local hit, d, n, s = QueryRaycast({x1, y1, z1}, {0, -1, 0}, y1)
        if hit then
            local x2 = rnd(player_pos[1] - rnd_xy, player_pos[1] + rnd_xy)
            local y2 = rnd(player_pos[2] - rnd_xy, player_pos[2] + rnd_xy)
            local z2 = rnd(player_pos[3] - rnd_xy, player_pos[3] + rnd_xy)

            add_thunderbolt({x1, y1, z1}, {x2, y2, z2})
        end

        next_hit = time + rnd(spawn_time_min, spawn_time_max)
    end

    for i = 1, #thunderbolts do
        local bolt = thunderbolts[i]

        -- decrease life
        if time >= prev_lifetime_time + 0.05 then
            bolt.life = bolt.life - 0.1
            local brightness = GetEnvironmentProperty("skyboxbrightness")
            local decreased = brightness
            decreased = decreased - (brightness) * 0.2

            if decreased > sky_brightness then
                SetEnvironmentProperty("skyboxbrightness", decreased)
            end
            prev_lifetime_time = time
        end

        if bolt.life <= 0 then
            table.remove(thunderbolts, i)
            SetEnvironmentProperty("skyboxbrightness", sky_brightness)
            break
        end

        -- draw the thunderbolt
        for j = 1, #bolt.lines do
            local start = bolt.lines[j].start
            local ending = bolt.lines[j].ending
            DrawLine(start, ending, 1, 1, 1, 1)

            PointLight(start, 1, 1, 1, bolt.life * light_multiplier)
            PointLight(ending, 1, 1, 1, bolt.life * light_multiplier)
        end

        -- draw the branches
        for j = 1, #bolt.branches do
            for k = 1, #bolt.branches[j] do
                local start = bolt.branches[j][k].start
                local ending = bolt.branches[j][k].ending
                DrawLine(start, ending, 1, 1, 1, 1)
                PointLight(start, 1, 1, 1, bolt.life)
                PointLight(ending, 1, 1, 1, bolt.life)
            end
        end
    end
end